#!/usr/bin/perl 
#
# fixbionetgen.pl
# written by Pedro Mendes, September 2017 - January 2018
#
# fixes the mass action rate laws of sbml produced with bionetgen
# adds annotation fields needed by pSSAlib

use POSIX;
use warnings;


if( $#ARGV < 0 ) { die "usage: fixbionetgensbml.pl sbmlfile\n"; }
# store command line arguments
$ifile = $ARGV[0];

open( IFILE, "$ifile" )  || die "can't open file $ifile";
$ofile = "fixed_$ifile";
open( OFILE, ">$ofile" ) || die "can't create file $ofile";

$lop = 0; # when we start we are not inside the list of parameters
$react = 0; # when we start we are not inside a reaction
my %params; # hash for parameter names

while( <IFILE> )
{
 $line = $_;
 $extraline = "";

 if( $line =~ /<listOfParameters>/ )
 {
  # we're inside the listOfParameters
  printf( OFILE "$line" );
  $lop = 1;
  next;
 }
 
 # change the species initial concentrations to initial amounts
 $line =~ s/initialConcentration/hasOnlySubstanceUnits=\"true\" initialAmount/ ;

 if( $line =~ /<\/listOfParameters>/ )
 {
  # we're no longer inside the listOfParameters
  $lop = 0;
  printf( OFILE "$line" );
  # debug output for my $pm (keys %params) {print " - $pm\n";}
  next;
 }
 
 if( $lop )
 {
  if( $line =~ /<parameter.*id="(.+?)"/)
  {
   # add this parameter to the hash
   $params{$1}=1;
  }
 }
 
# find model id and fix it!
 if(  $line =~ /<model id=\"(.+)\">/ )
 {
  $line = "<model id=\"model1\" name=\"$1\">\n";
  $extraline = "<listOfUnitDefinitions><unitDefinition id=\"substance\" name=\"substance\"><listOfUnits><unit kind=\"item\" exponent=\"1\" scale=\"0\" multiplier=\"1\"/></listOfUnits></unitDefinition></listOfUnitDefinitions>\n";
 }
 
# find compartment and fix its size  
# if(  $line =~ /<compartment id=\"cell\" size=\"1\"\/>/ )
# {
#  $line =~ s/1/4e-14/;
# }

# make sure we know we are inside a reaction element
 if( $line =~ /<reaction\s/ )
 {
  # we're inside a reaction
  $react = 1;
  # we don't know which parameter is this one...
  $reactp = undef;
 }

# check each <ci> </ci> element to see if it is a parameter
# WARNING: if the rate law has more than one parameter we only keep the last one...
 if( $react )
 {
  if( $line =~ /<ci>\s*(\S+?)\s*<\/ci>/ )
  {
   if( $params{$1} ) # the value is a key in the hash
   {
    $reactp = $1;
   }
  }
 }

# detect end of a reaction block
 if( $line =~ /<\/reaction>/ )
 {
  # we've finished with this reaction
  $react = 0;
  # make annotation needed for pSSAlib
  $line = " <annotation>\n  <libpSSA:rate xmlns:libpSSA=\"uri\">\n    <libpSSA:forward libpSSA:value=\"$reactp\"/>\n  </libpSSA:rate>\n </annotation>\n";
  # and end the reaction
  $extraline = "</reaction>\n";
 }


# find rate law 
# we assume here that all reactions in the SBML are mass action
# WARNING: this code will fail if applied to an SBML file that has other rate laws!
 if( $line =~ /<times\/>/ )
 {
  $extraline = "              <ci> cell <\/ci>\n";
 }
 printf( OFILE "$line$extraline" );
}
close( IFILE );
close( OFILE );
