setwd("~/CQM/Projects/RBS/Examples/Models/Analysis")
source("helperFunctions.R")

########################################################################
################## For scaling of execution times Molecules ##############
#########################################################################


timingResults= read.table("Results/Multistate_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res1= data.frame(formatResults(timingResults),Model="Multi-state model")

timingResults= read.table("Results/Multisite_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res2= data.frame(formatResults(timingResults),Model="Multi-site model")

result= rbind(res1, res2)
p1 <- qplot(Fold,log10(mVal),data=result,pch= Simulators,color= Simulators,geom="line",xlab="# Molecules at start",ylab = expression(paste("log"[10],"(Execution time)"))) + scale_x_continuous(label=scientific,breaks = scales::pretty_breaks(n = 2.5)) + annotate("text", x = 1000, y = 3.25, label = "A") + scale_color_hue(limits=levels(result$id),l=40, c=80) +geom_point() + theme_classic() + facet_wrap( ~ Model,scales = "free_x", strip.position="bottom") + scale_shape_manual(values=unique(result$id))  + ylim(-2,3.25)


timingResults= read.table("Results/EGFR_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res3= data.frame(formatResults(timingResults),Model="EGFR model")

timingResults= read.table("Results/BCR_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res4= data.frame(formatResults(timingResults),Model="BCR model")

timingResults= read.table("Results/Fceri_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res5= data.frame(formatResults(timingResults),Model="FcεRI model")

result= rbind(res3,res4,res5)


p2 <- qplot(Fold,log10(mVal),data=result,pch= Simulators,color= Simulators,geom="line",xlab="# Molecules at start",ylab = expression(paste("log"[10],"(Execution time)"))) + scale_x_continuous(label=scientific,breaks = scales::pretty_breaks(n = 2.5)) + annotate("text", x = 1000, y = 3.65, label = "A") + scale_color_hue(limits=levels(result$id),l=40, c=80) +geom_point() + theme_classic() + facet_wrap( ~ Model,scales = "free_x", strip.position="bottom") + scale_shape_manual(values=unique(result$id))  + ylim(-1.2,3.65)


pdf("Molecules.pdf",width = 8,height = 8)
grid.arrange(p1,p2,ncol=1)
dev.off()



########################################################################
################## For scaling of execution times End Time ##############
#########################################################################


timingResults= read.table("Results/Multistate_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res1= data.frame(formatResults(timingResults),Model="Multi-state model")

timingResults= read.table("Results/Multisite_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res2= data.frame(formatResults(timingResults),Model="Multi-site model")

result= rbind(res1, res2)
p1 <- qplot(Fold,log10(mVal),data=result,pch= Simulators,color= Simulators,geom="line",xlab="Simulation End Time (s)",ylab = expression(paste("log"[10],"(Execution time)"))) + scale_color_hue(limits=levels(result$id),l=40, c=80) +geom_point() + theme_classic() + facet_wrap( ~ Model,scales = "free_x", strip.position="bottom") + scale_shape_manual(values=unique(result$id)) + annotate("text", x = 10, y = 3.35, label = "A") + ylim(-2,3.35)

timingResults= read.table("Results/EGFR_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res3= data.frame(formatResults(timingResults),Model="EGFR model")

timingResults= read.table("Results/BCR_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res4= data.frame(formatResults(timingResults),Model="BCR model")

timingResults= read.table("Results/Fceri_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res5= data.frame(formatResults(timingResults),Model="FcεRI model")

result= rbind(res3,res4,res5)
p2<- qplot(Fold,log10(mVal),data=result,pch= Simulators,color= Simulators,geom="line",xlab="Simulation End Time (s)",ylab = expression(paste("log"[10],"(Execution time)"))) + scale_color_hue(limits=levels(result$id),l=40, c=80) +geom_point() + theme_classic() + facet_wrap( ~ Model,scales = "free_x", strip.position="bottom") + scale_shape_manual(values=unique(result$id)) + annotate("text", x = 10, y = 3.65, label = "A")


pdf("Times.pdf",width = 8,height = 8)
grid.arrange(p1,p2,ncol=1)
dev.off()




########################################################################
################## For Random number generator comparison ##############
################## ######################################################
randNum= read.table("Random_C.txt",stringsAsFactors = F,header = T)
randMT= read.table("Random_MT.txt",stringsAsFactors = F,header = T)

res1= randNum %>%
  group_by(Number) %>%
  summarise(mVal = median(Time,na.rm=T))

res2= randMT %>%
  group_by(Number) %>%
  summarise(mVal = median(Time,na.rm=T))

result= data.frame(Number= log10(res1$Number),C_inbuilt=res1$mVal, Mersenne_Twister=res2$mVal)
result= melt(result,id="Number")
colnames(result)[2]= "Method"
p1<- ggplot(result, aes(Number,value)) + geom_line(aes(linetype=Method)) + guides(fill=guide_legend(title=NULL)) + theme_classic() + labs(x=expression(log[10]*"(# random numbers)"), y= "Time (s)")


randBNGL= read.table("Results/RND_BioNetGen.csv",sep=",",stringsAsFactors = F,header = T)

result= randBNGL %>%
  gather(var, val, 3:4) %>%
  group_by(Endtime, var) %>%
  summarise(mVal = median(val,na.rm=T))
colnames(result)[2]= "Simulators"

p2 <- qplot(Endtime,log10(mVal),data=result,pch= Simulators,linetype= Simulators,geom="line",xlab="Simulation End Time (s)",ylab = expression(paste("log"[10],"(Execution time)")))  +geom_point() + theme_classic()

pdf(file = "Rnd.pdf",width = 10, height=4)
grid.arrange(p1,p2,ncol=2)
dev.off()




########################################################################
################## Summary results showing fastest simulator ##############
################## ######################################################

timingResults= read.table("Results/Multistate_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res1= data.frame(transformData(timingResults),Model="Multi-state")

timingResults= read.table("Results/Multisite_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res2= data.frame(transformData(timingResults),Model="Multi-site")

timingResults= read.table("Results/EGFR_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res3= data.frame(transformData(timingResults),Model="EGFR")

timingResults= read.table("Results/BCR_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res4= data.frame(transformData(timingResults),Model="BCR")
res4$Fold= res4$Fold/5

timingResults= read.table("Results/Fceri_results_molecules.csv",sep=",",header = T,stringsAsFactors = F)
res5= data.frame(transformData(timingResults),Model="FcεRI")

result1= rbind(res1,res2,res3,res4,res5)
results= data.frame(result1,Test="# Molecules at the start")


timingResults= read.table("Results/Multistate_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res1= data.frame(transformData(timingResults),Model="Multi-state")

timingResults= read.table("Results/Multisite_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res2= data.frame(transformData(timingResults),Model="Multi-site")

timingResults= read.table("Results/EGFR_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res3= data.frame(transformData(timingResults),Model="EGFR")

timingResults= read.table("Results/BCR_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res4= data.frame(transformData(timingResults),Model="BCR")

timingResults= read.table("Results/Fceri_results_time.csv",sep=",",header = T,stringsAsFactors = F)
res5= data.frame(transformData(timingResults),Model="FcεRI")

result2= rbind(res1,res2,res3,res4,res5)
results= rbind(results,data.frame(result2,Test="Simulation End Time"))


pdf("Overall_res.pdf",width = 8,height = 4)
qplot(log10(Fold),Model,data=results,pch= Simulators,color= Simulators,ylab="",xlab = "") + scale_color_hue(limits=levels(results$id),l=40, c=80) + geom_point(size=2) + theme_classic()  + scale_shape_manual(values=unique(results$id)) + annotate("text", x = 2, y = 5.25, label = "A") + facet_wrap( ~ Test,scales = "free_x", strip.position="bottom")
dev.off()
